/*
 * Copyright  2007, The MorphOS Team. All rights reserved.
 *
 * Edited by walkero
 *
 * $Id: greek.c,v 1.8 2012/01/08 00:45:02 tokai Exp $
 */

/// includes
#define __NOLIBBASE__
#define USE_INLINE_STDARG

#include <exec/resident.h>
#include <exec/libraries.h>
#include <libraries/locale.h>
#include <proto/exec.h>
#include <proto/dos.h>
#include <proto/locale.h>

#include "language.h"

#define LIBNAME "greek.language"
#define VERSION         51
#define REVISION        3
#define DATE    __AMIGADATE__
#define VERS    LIBNAME " 51.3"
#define VSTRING VERS " (" DATE ")  "__COPYRIGHTYEAR__" The MorphOS Team\r\n"
#define VERSTAG "\0$VER: " VSTRING

struct LanguageBase
{
	struct Library          LibNode;
	struct ExecBase         *SysBase;
	APTR                    Seglist;
	struct SignalSemaphore  SemLock;
};

struct LocaleBase *LocaleBase;

#define LF_MASK (LF_ConvToLower | LF_ConvToUpper | LF_GetLangStr | LF_IsAlNum | LF_IsAlpha | LF_IsCntrl | LF_IsDigit | LF_IsGraph | LF_IsLower | LF_IsPrint | LF_IsPunct | LF_IsSpace | LF_IsUpper | LF_IsXDigit | LF_ToUpper | LF_StringComp | LF_GetLangStrU)

#define MKBYTE(c,b) (((c) & ~0xFF) | (b))
#define MKBOOL(c,b) (((c) & ~0xFF) | ((b) ? 0xFF : 0))

/* standards */

struct Library *LibInit(struct Library *unused, APTR seglist, struct Library *sysb);
APTR lib_expunge(struct LanguageBase *cb);
struct Library *LibOpen(void);
ULONG LibClose(void);
APTR LibExpunge(void);
ULONG LibReserved(void);
LONG dummy_function(void);
BOOL InitResources(struct LanguageBase *cb);

static const CONST_WSTRPTR GreekStringsU[];
static const CONST_STRPTR GreekStrings[];
static const UWORD LocalizedCTypes[256];
static const UBYTE LangConvToUpper[256];
static const UBYTE LangConvToLower[256];
static const UBYTE GreekCollate1[256];
static const UBYTE GreekCollate2[256];

/* specific */

ULONG LibLanguageMask(void);
CONST_STRPTR LibGetLangString(void);
ULONG LibConvToLower(void);
ULONG LibConvToUpper(void);
ULONG LibIsAlnum(void);
ULONG LibIsAlpha(void);
ULONG LibIsCntrl(void);
ULONG LibIsDigit(void);
ULONG LibIsGraph(void);
ULONG LibIsLower(void);
ULONG LibIsPrint(void);
ULONG LibIsPunct(void);
ULONG LibIsSpace(void);
ULONG LibIsUpper(void);
ULONG LibIsXDigit(void);
LONG LibStrCompare(void);
CONST_WSTRPTR LibGetLangStringU(ULONG id, struct LocaleBase *LocaleBase);

const UBYTE LibName[] = LIBNAME;
const UBYTE VTag[] = VERSTAG;

const struct Resident __attribute__((section(".text"))) ROMTag =
{
	RTC_MATCHWORD,
	(APTR)&ROMTag,
	(APTR)&ROMTag + 1,
	RTF_EXTENDED | RTF_PPC,
	VERSION,
	NT_LIBRARY,
	-120,
	(STRPTR)LibName,
	VSTRING,
	(APTR)LibInit
	, REVISION,
	NULL
};

const CONST_APTR JumpTable[] =
{
	(CONST_APTR) FUNCARRAY_BEGIN,
	(APTR)FUNCARRAY_32BIT_NATIVE,
	(APTR)LibOpen,
	(APTR)LibClose,
	(APTR)LibExpunge,
	(APTR)LibReserved,
	(APTR)LibLanguageMask,
	(APTR)LibConvToLower,
	(APTR)LibConvToUpper,
	(APTR)LibReserved,      // Private2 (Undocumented? Reserved?)
	(APTR)LibGetLangString,
	(APTR)LibIsAlnum,
	(APTR)LibIsAlpha,
	(APTR)LibIsCntrl,
	(APTR)LibIsDigit,
	(APTR)LibIsGraph,
	(APTR)LibIsLower,
	(APTR)LibIsPrint,
	(APTR)LibIsPunct,
	(APTR)LibIsSpace,
	(APTR)LibIsUpper,
	(APTR)LibIsXDigit,
	(APTR)LibReserved,      // for StrConvert
	(APTR)LibStrCompare,
	(APTR)0xFFFFFFFF,

	/* 17 */
	(APTR) FUNCARRAY_32BIT_SYSTEMV,
	&LibGetLangStringU,
	(APTR) - 1L,

	(CONST_APTR) FUNCARRAY_END
};

///
/// LibInit()

static struct LanguageBase *lib_init(struct LanguageBase *cb, APTR seglist, struct ExecBase *sysbase)
{
	struct ExecBase *SysBase = sysbase;
	cb->SysBase = sysbase;
	cb->Seglist = seglist;

	InitSemaphore(&cb->SemLock);

	return cb;
}

struct Library *LibInit(struct Library *unused, APTR seglist, struct Library *sysbase)
{
	struct ExecBase *SysBase;
	unused = unused;      /* suppress warning */
	SysBase = (APTR)sysbase;

	return (NewCreateLibraryTags(
	            LIBTAG_FUNCTIONINIT, (ULONG)JumpTable,
	            LIBTAG_LIBRARYINIT, (ULONG)lib_init,
	            LIBTAG_MACHINE,      MACHINE_PPC,
	            LIBTAG_BASESIZE,     sizeof(struct LanguageBase),
	            LIBTAG_SEGLIST, (ULONG)seglist,
	            LIBTAG_TYPE,         NT_LIBRARY,
	            LIBTAG_PRI,          -120,
	            LIBTAG_NAME, (ULONG)ROMTag.rt_Name,
	            LIBTAG_IDSTRING, (ULONG)ROMTag.rt_IdString,
	            LIBTAG_FLAGS,        LIBF_CHANGED | LIBF_SUMUSED,
	            LIBTAG_VERSION,      VERSION,
	            LIBTAG_REVISION,     REVISION,
	            LIBTAG_PUBLIC,       TRUE,
	            TAG_END));
}

///
/// LibOpen()

struct Library *LibOpen(void)
{
	struct LanguageBase *cb = (struct LanguageBase*)REG_A6;
	struct Library *lib = (struct Library*)cb;
	struct ExecBase *SysBase = cb->SysBase;
	
	ObtainSemaphore(&cb->SemLock);
	if(!LocaleBase)
	{
		struct ExecBase *SysBase = cb->SysBase;

		LocaleBase = (struct LocaleBase *)OpenLibrary("locale.library", 51);

		if(!LocaleBase)
			lib = NULL;
	}
	ReleaseSemaphore(&cb->SemLock);

#warning this is possibly unprotected, should move to start of function. In failure case decreased again.
	if(lib)
	{
		cb->LibNode.lib_Flags &= ~LIBF_DELEXP;
		cb->LibNode.lib_OpenCnt++;
	}

	return lib;
}

///
/// LibClose()

ULONG LibClose(void)
{
	struct LanguageBase *cb = (struct LanguageBase*)REG_A6;
	struct ExecBase *SysBase = cb->SysBase;
	ULONG ret = 0;

	ObtainSemaphore(&cb->SemLock);
	if(--cb->LibNode.lib_OpenCnt == 0)
	{
		struct ExecBase *SysBase = cb->SysBase;

		if(LocaleBase) CloseLibrary((struct Library *)LocaleBase);
		LocaleBase = NULL;

		if(cb->LibNode.lib_Flags & LIBF_DELEXP)
			ret = (ULONG)lib_expunge(cb);
	}
	ReleaseSemaphore(&cb->SemLock);

	return ret;
}

///
/// LibExpunge()

APTR LibExpunge(void)
{
	struct LanguageBase *cb = (struct LanguageBase*)REG_A6;

	return(lib_expunge(cb));
}


APTR lib_expunge(struct LanguageBase *cb)
{
	APTR seglist = NULL;

	if(cb->LibNode.lib_OpenCnt == 0)
	{
		struct ExecBase *SysBase = cb->SysBase;
		REMOVE((struct Node*)cb);
		seglist = cb->Seglist;
		FreeMem((UBYTE*)cb - cb->LibNode.lib_NegSize, cb->LibNode.lib_NegSize + cb->LibNode.lib_PosSize);
		cb = NULL;    /* freed memory, no more valid */
	}
	else cb->LibNode.lib_Flags |= LIBF_DELEXP;

	return seglist;
}

///
/// LibReserved()

ULONG LibReserved(void)
{
	return 0;
}

ULONG LibLanguageMask(void)
{
	return LF_MASK;
}

CONST_WSTRPTR LibGetLangStringU(ULONG id, struct LocaleBase *LocaleBase)
{
	if(id < MAXSTRMSG)
		return GreekStringsU[id];
	else
		return NULL;
}

CONST_STRPTR LibGetLangString(void)
{
	ULONG id = REG_D0;

	if(id < MAXSTRMSG) return GreekStrings[id];
	else return NULL;
}

ULONG LibConvToLower(void)
{
	ULONG chr = REG_D0;

	return MKBYTE(chr, LangConvToLower[(UBYTE)chr]);
}

ULONG LibConvToUpper(void)
{
	ULONG chr = REG_D0;

	return MKBYTE(chr, LangConvToUpper[(UBYTE)chr]);
}

///
/// LibIsAlnum()

ULONG LibIsAlnum(void)
{
	ULONG chr = REG_D0;

	return MKBOOL(chr, LocalizedCTypes[(UBYTE)chr] & iAlNum);
}

///
/// LibIsAlpha()

ULONG LibIsAlpha(void)
{
	ULONG chr = REG_D0;

	return MKBOOL(chr, LocalizedCTypes[(UBYTE)chr] & iAlpha);
}

///
/// LibIsCntrl()

ULONG LibIsCntrl(void)
{
	ULONG chr = REG_D0;

	return MKBOOL(chr, LocalizedCTypes[(UBYTE)chr] & iCntrl);
}

///
/// LibIsDigit()

ULONG LibIsDigit(void)
{
	ULONG chr = REG_D0;

	return MKBOOL(chr, LocalizedCTypes[(UBYTE)chr] & iDigit);
}

///
/// LibIsGraph()

ULONG LibIsGraph(void)
{
	ULONG chr = REG_D0;

	return MKBOOL(chr, LocalizedCTypes[(UBYTE)chr] & iGraph);
}

///
/// LibIsLower()

ULONG LibIsLower(void)
{
	ULONG chr = REG_D0;

	return MKBOOL(chr, LocalizedCTypes[(UBYTE)chr] & iLower);
}

///
/// LibIsPrint()

ULONG LibIsPrint(void)
{
	ULONG chr = REG_D0;

	return MKBOOL(chr, LocalizedCTypes[(UBYTE)chr] & iPrint);
}

///
/// LibIsPunct()

ULONG LibIsPunct(void)
{
	ULONG chr = REG_D0;

	return MKBOOL(chr, LocalizedCTypes[(UBYTE)chr] & iPunct);
}

///
/// LibIsSpace()

ULONG LibIsSpace(void)
{
	ULONG chr = REG_D0;

	return MKBOOL(chr, LocalizedCTypes[(UBYTE)chr] & iSpace);
}

///
/// LibIsUpper()

ULONG LibIsUpper(void)
{
	ULONG chr = REG_D0;

	return MKBOOL(chr, LocalizedCTypes[(UBYTE)chr] & iUpper);
}

///
/// LibIsXDigit()

ULONG LibIsXDigit(void)
{
	ULONG chr = REG_D0;

	return MKBOOL(chr, LocalizedCTypes[(UBYTE)chr] & iXDigit);
}

///
/// LibStrCompare()

LONG LibStrCompare(void)
{

	STRPTR string1 = (STRPTR)REG_A1;
	STRPTR string2 = (STRPTR)REG_A2;
	LONG length = REG_D0;
	ULONG type = REG_D1;

	CONST UBYTE *coll_tab;

	switch(type)
	{
	case SC_ASCII:
		coll_tab = LangConvToUpper;
		break;
	case SC_COLLATE1:
		coll_tab = GreekCollate1;
		break;
	case SC_COLLATE2:
		coll_tab = GreekCollate2;
		break;
	default:
		return 0;
	}

	return StrCompareInternal(string1, string2, length, coll_tab, LocalizedCTypes);
}

#warning !!!!!!!!!!!!!!!!! THIS IS COMPLETELY BROKEN !!!!!!!!!!!!!!!!!!!!!!!!

/* XXX: This is broken. Greek doesn't use latin1, so those strings/arrays don't map properly to the right
 *      unicode points. Needs to be fixed! 
 *
 *      -- tokai, January 8, 2012
 */

static const CONST_WSTRPTR GreekStringsU[] =
{
	/* A blank string */

	(CONST_WSTRPTR)L"",

	/*  The days of the week. Starts with the first day of the week.
	        In English this would be Sunday, this depends upon the settings
	        of Locale->CalendarType. */

	(CONST_WSTRPTR)L"", (CONST_WSTRPTR)L"", (CONST_WSTRPTR)L"", (CONST_WSTRPTR)L"", (CONST_WSTRPTR)L"", (CONST_WSTRPTR)L"",
	(CONST_WSTRPTR)L"",

	/* Abbreviated days of the week */

	(CONST_WSTRPTR)L"", (CONST_WSTRPTR)L"", (CONST_WSTRPTR)L"", (CONST_WSTRPTR)L"", (CONST_WSTRPTR)L"", (CONST_WSTRPTR)L"", (CONST_WSTRPTR)L"",

	/* Months of the year */

	(CONST_WSTRPTR)L"", (CONST_WSTRPTR)L"", (CONST_WSTRPTR)L"", (CONST_WSTRPTR)L"", (CONST_WSTRPTR)L"", (CONST_WSTRPTR)L"",
	(CONST_WSTRPTR)L"", (CONST_WSTRPTR)L"", (CONST_WSTRPTR)L"", (CONST_WSTRPTR)L"", (CONST_WSTRPTR)L"", (CONST_WSTRPTR)L"",

	/* Abbreviated months of the year */

	(CONST_WSTRPTR)L"", (CONST_WSTRPTR)L"", (CONST_WSTRPTR)L"", (CONST_WSTRPTR)L"", (CONST_WSTRPTR)L"", (CONST_WSTRPTR)L"",
	(CONST_WSTRPTR)L"", (CONST_WSTRPTR)L"", (CONST_WSTRPTR)L"", (CONST_WSTRPTR)L"", (CONST_WSTRPTR)L"", (CONST_WSTRPTR)L"",

	(CONST_WSTRPTR)L"", /* Yes, affirmative response */
	(CONST_WSTRPTR)L"", /* No/negative response */

	/* AM/PM strings AM 0000 -> 1159, PM 1200 -> 2359 */

	(CONST_WSTRPTR)L"", (CONST_WSTRPTR)L"",

	/* Soft and hard hyphens */

	(CONST_WSTRPTR)L"-", (CONST_WSTRPTR)L"-",

	/* Open and close quotes */

	(CONST_WSTRPTR)L"\"", (CONST_WSTRPTR)L"\"",

	/* Days: But not actual day names
	   Yesterday - the day before the current
	   Today - the current day
	   Tomorrow - the next day
	   Future.
	*/

	(CONST_WSTRPTR)L"", (CONST_WSTRPTR)L"", (CONST_WSTRPTR)L"", (CONST_WSTRPTR)L""
};

///
/// GreekStrings[]

static const CONST_STRPTR GreekStrings[] =
{
	/* A blank string */

	"",

	/*  The days of the week. Starts with the first day of the week.
	        In English this would be Sunday, this depends upon the settings
	        of Locale->CalendarType. */

	"", "", "", "", "", "", "",

	/* Abbreviated days of the week */

	"", "", "", "", "", "", "",

	/* Months of the year */

	"", "", "", "", "", "",
	"", "", "", "", "", "",

	/* Abbreviated months of the year */

	"", "", "", "", "", "",
	"", "", "", "", "", "",

	"", /* Yes, affirmative response */
	"", /* No/negative response */

	/* AM/PM strings AM 0000 -> 1159, PM 1200 -> 2359 */

	"", "",

	/* Soft and hard hyphens */

	"-", "-",

	/* Open and close quotes */

	"\"", "\"",

	/* Days: But not actual day names
	   Yesterday - the day before the current
	   Today - the current day
	   Tomorrow - the next day
	   Future.
	*/

	"", "", "", ""
};

static const UWORD LocalizedCTypes[256] =
{
	iCntrl, /* NULL */
	iCntrl, /* START OF HEADING */
	iCntrl, /* START OF TEXT */
	iCntrl, /* END OF TEXT */
	iCntrl, /* END OF TRANSMISSION */
	iCntrl, /* ENQUIRY */
	iCntrl, /* ACKNOWLEDGE */
	iCntrl, /* BELL */
	iCntrl | iPrint | iSpace, /* BACKSPACE */
	iCntrl | iPrint | iSpace, /* HORIZONTAL TABULATION */
	iCntrl | iPrint | iSpace, /* LINE FEED */
	iCntrl | iPrint | iSpace, /* VERTICAL TABULATION */
	iCntrl | iPrint | iSpace, /* FORM FEED */
	iCntrl | iPrint | iSpace, /* CARRIAGE RETURN */
	iCntrl, /* SHIFT OUT */
	iCntrl, /* SHIFT IN */
	iCntrl, /* DATA LINK ESCAPE */
	iCntrl, /* DEVICE CONTROL ONE */
	iCntrl, /* DEVICE CONTROL TWO */
	iCntrl, /* DEVICE CONTROL THREE */
	iCntrl, /* DEVICE CONTROL FOUR */
	iCntrl, /* NEGATIVE ACKNOWLEDGE */
	iCntrl, /* SYNCHRONOUS IDLE */
	iCntrl, /* END OF TRANSMISSION BLOCK */
	iCntrl, /* CANCEL */
	iCntrl, /* END OF MEDIUM */
	iCntrl, /* SUBSTITUTE */
	iCntrl, /* ESCAPE */
	iCntrl, /* FILE SEPARATOR */
	iCntrl, /* GROUP SEPARATOR */
	iCntrl, /* RECORD SEPARATOR */
	iCntrl, /* UNIT SEPARATOR */

	iPrint | iSpace, /* SPACE */
	iGraph | iPrint | iPunct, /* EXCLAMATION MARK */
	iGraph | iPrint | iPunct, /* QUOTATION MARK */
	iGraph | iPrint | iPunct, /* NUMBER SIGN */
	iGraph | iPrint | iPunct, /* DOLLAR SIGN */
	iGraph | iPrint | iPunct, /* PERCENT SIGN */
	iGraph | iPrint | iPunct, /* AMPERSAND */
	iGraph | iPrint | iPunct, /* APOSTROPHE */
	iGraph | iPrint | iPunct, /* LEFT PARENTHESIS */
	iGraph | iPrint | iPunct, /* RIGHT PARENTHESIS */
	iGraph | iPrint | iPunct, /* ASTERISK */
	iGraph | iPrint | iPunct, /* PLUS SIGN */
	iGraph | iPrint | iPunct, /* COMMA */
	iGraph | iPrint | iPunct, /* HYPHEN-MINUS */
	iGraph | iPrint | iPunct, /* FULL STOP */
	iGraph | iPrint | iPunct, /* SOLIDUS */
	iGraph | iPrint | iDigit | iXDigit, /* DIGIT ZERO */
	iGraph | iPrint | iDigit | iXDigit, /* DIGIT ONE */
	iGraph | iPrint | iDigit | iXDigit, /* DIGIT TWO */
	iGraph | iPrint | iDigit | iXDigit, /* DIGIT THREE */
	iGraph | iPrint | iDigit | iXDigit, /* DIGIT FOUR */
	iGraph | iPrint | iDigit | iXDigit, /* DIGIT FIVE */
	iGraph | iPrint | iDigit | iXDigit, /* DIGIT SIX */
	iGraph | iPrint | iDigit | iXDigit, /* DIGIT SEVEN */
	iGraph | iPrint | iDigit | iXDigit, /* DIGIT EIGHT */
	iGraph | iPrint | iDigit | iXDigit, /* DIGIT NINE */
	iGraph | iPrint | iPunct, /* COLON */
	iGraph | iPrint | iPunct, /* SEMICOLON */
	iGraph | iPrint | iPunct, /* LESS-THAN SIGN */
	iGraph | iPrint | iPunct, /* EQUALS SIGN */
	iGraph | iPrint | iPunct, /* GREATER-THAN SIGN */
	iGraph | iPrint | iPunct, /* QUESTION MARK */

	iGraph | iPrint | iPunct, /* COMMERCIAL AT */
	iAlpha | iGraph | iPrint | iUpper | iXDigit, /* LATIN CAPITAL LETTER A */
	iAlpha | iGraph | iPrint | iUpper | iXDigit, /* LATIN CAPITAL LETTER B */
	iAlpha | iGraph | iPrint | iUpper | iXDigit, /* LATIN CAPITAL LETTER C */
	iAlpha | iGraph | iPrint | iUpper | iXDigit, /* LATIN CAPITAL LETTER D */
	iAlpha | iGraph | iPrint | iUpper | iXDigit, /* LATIN CAPITAL LETTER E */
	iAlpha | iGraph | iPrint | iUpper | iXDigit, /* LATIN CAPITAL LETTER F */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER G */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER H */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER I */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER J */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER K */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER L */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER M */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER N */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER O */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER P */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER Q */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER R */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER S */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER T */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER U */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER V */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER W */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER X */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER Y */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER Z */
	iGraph | iPrint | iPunct, /* LEFT SQUARE BRACKET */
	iGraph | iPrint | iPunct, /* REVERSE SOLIDUS */
	iGraph | iPrint | iPunct, /* RIGHT SQUARE BRACKET */
	iGraph | iPrint | iPunct, /* CIRCUMFLEX ACCENT */
	iGraph | iPrint | iPunct, /* LOW LINE */

	iGraph | iPrint | iPunct, /* GRAVE ACCENT */
	iAlpha | iGraph | iLower | iPrint | iXDigit, /* LATIN SMALL LETTER A */
	iAlpha | iGraph | iLower | iPrint | iXDigit, /* LATIN SMALL LETTER B */
	iAlpha | iGraph | iLower | iPrint | iXDigit, /* LATIN SMALL LETTER C */
	iAlpha | iGraph | iLower | iPrint | iXDigit, /* LATIN SMALL LETTER D */
	iAlpha | iGraph | iLower | iPrint | iXDigit, /* LATIN SMALL LETTER E */
	iAlpha | iGraph | iLower | iPrint | iXDigit, /* LATIN SMALL LETTER F */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER G */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER H */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER I */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER J */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER K */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER L */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER M */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER N */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER O */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER P */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER Q */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER R */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER S */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER T */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER U */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER V */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER W */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER X */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER Y */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER Z */
	iGraph | iPrint | iPunct, /* LEFT CURLY BRACKET */
	iGraph | iPrint | iPunct, /* VERTICAL LINE */
	iGraph | iPrint | iPunct, /* RIGHT CURLY BRACKET */
	iGraph | iPrint | iPunct, /* TILDE */
	iCntrl, /* DELETE */

	iCntrl, /* <control> */
	iCntrl, /* <control> */
	iCntrl, /* <control> */
	iCntrl, /* <control> */
	iCntrl, /* <control> */
	iCntrl, /* <control> */
	iCntrl, /* <control> */
	iCntrl, /* <control> */
	iCntrl, /* <control> */
	iCntrl, /* <control> */
	iCntrl, /* <control> */
	iCntrl, /* <control> */
	iCntrl, /* <control> */
	iCntrl, /* <control> */
	iCntrl, /* <control> */
	iCntrl, /* <control> */
	iCntrl, /* <control> */
	iCntrl, /* <control> */
	iCntrl, /* <control> */
	iCntrl, /* <control> */
	iCntrl, /* <control> */
	iCntrl, /* <control> */
	iCntrl, /* <control> */
	iCntrl, /* <control> */
	iCntrl, /* <control> */
	iCntrl, /* <control> */
	iCntrl, /* <control> */
	iCntrl, /* <control> */
	iCntrl, /* <control> */
	iCntrl, /* <control> */
	iCntrl, /* <control> */
	iCntrl, /* <control> */

	iPrint | iPunct, /* NO-BREAK SPACE */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER A WITH OGONEK */
	iGraph | iPrint, /* BREVE */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER L WITH STROKE */
	iGraph | iPrint | iPunct, /* CURRENCY SIGN */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER L WITH CARON */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER S WITH ACUTE */
	iGraph | iPrint | iPunct, /* SECTION SIGN */
	iGraph | iPrint, /* DIAERESIS */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER S WITH CARON */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER S WITH CEDILLA */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER T WITH CARON */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER Z WITH ACUTE */
	iGraph | iPrint | iPunct, /* SOFT HYPHEN */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER Z WITH CARON */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER Z WITH DOT ABOVE */
	iGraph | iPrint | iPunct, /* DEGREE SIGN */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER A WITH OGONEK */
	iGraph | iPrint, /* OGONEK */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER L WITH STROKE */
	iGraph | iPrint, /* ACUTE ACCENT */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER L WITH CARON */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER S WITH ACUTE */
	iGraph | iPrint, /* CARON */
	iGraph | iPrint, /* CEDILLA */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER S WITH CARON */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER S WITH CEDILLA */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER T WITH CARON */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER Z WITH ACUTE */
	iGraph | iPrint, /* DOUBLE ACUTE ACCENT */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER Z WITH CARON */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER Z WITH DOT ABOVE */

	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER R WITH ACUTE */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER A WITH ACUTE */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER A WITH CIRCUMFLEX */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER A WITH BREVE */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER A WITH DIAERESIS */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER L WITH ACUTE */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER C WITH ACUTE */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER C WITH CEDILLA */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER C WITH CARON */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER E WITH ACUTE */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER E WITH OGONEK */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER E WITH DIAERESIS */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER E WITH CARON */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER I WITH ACUTE */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER I WITH CIRCUMFLEX */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER D WITH CARON */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER D WITH STROKE */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER N WITH ACUTE */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER N WITH CARON */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER O WITH ACUTE */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER O WITH CIRCUMFLEX */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER O WITH DOUBLE ACUTE */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER O WITH DIAERESIS */
	iGraph | iPrint | iPunct, /* MULTIPLICATION SIGN */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER R WITH CARON */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER U WITH RING ABOVE */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER U WITH ACUTE */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER U WITH DOUBLE ACUTE */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER U WITH DIAERESIS */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER Y WITH ACUTE */
	iAlpha | iGraph | iPrint | iUpper, /* LATIN CAPITAL LETTER T WITH CEDILLA */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER SHARP S */

	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER R WITH ACUTE */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER A WITH ACUTE */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER A WITH CIRCUMFLEX */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER A WITH BREVE */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER A WITH DIAERESIS */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER L WITH ACUTE */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER C WITH ACUTE */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER C WITH CEDILLA */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER C WITH CARON */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER E WITH ACUTE */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER E WITH OGONEK */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER E WITH DIAERESIS */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER E WITH CARON */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER I WITH ACUTE */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER I WITH CIRCUMFLEX */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER D WITH CARON */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER D WITH STROKE */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER N WITH ACUTE */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER N WITH CARON */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER O WITH ACUTE */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER O WITH CIRCUMFLEX */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER O WITH DOUBLE ACUTE */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER O WITH DIAERESIS */
	iGraph | iPrint | iPunct, /* DIVISION SIGN */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER R WITH CARON */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER U WITH RING ABOVE */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER U WITH ACUTE */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER U WITH DOUBLE ACUTE */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER U WITH DIAERESIS */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER Y WITH ACUTE */
	iAlpha | iGraph | iLower | iPrint, /* LATIN SMALL LETTER T WITH CEDILLA */
	iGraph | iPrint, /* DOT ABOVE */
};

static const UBYTE LangConvToLower[256] =
{
	0x00, /* NULL */
	0x01, /* START OF HEADING */
	0x02, /* START OF TEXT */
	0x03, /* END OF TEXT */
	0x04, /* END OF TRANSMISSION */
	0x05, /* ENQUIRY */
	0x06, /* ACKNOWLEDGE */
	0x07, /* BELL */
	0x08, /* BACKSPACE */
	0x09, /* HORIZONTAL TABULATION */
	0x0A, /* LINE FEED */
	0x0B, /* VERTICAL TABULATION */
	0x0C, /* FORM FEED */
	0x0D, /* CARRIAGE RETURN */
	0x0E, /* SHIFT OUT */
	0x0F, /* SHIFT IN */
	0x10, /* DATA LINK ESCAPE */
	0x11, /* DEVICE CONTROL ONE */
	0x12, /* DEVICE CONTROL TWO */
	0x13, /* DEVICE CONTROL THREE */
	0x14, /* DEVICE CONTROL FOUR */
	0x15, /* NEGATIVE ACKNOWLEDGE */
	0x16, /* SYNCHRONOUS IDLE */
	0x17, /* END OF TRANSMISSION BLOCK */
	0x18, /* CANCEL */
	0x19, /* END OF MEDIUM */
	0x1A, /* SUBSTITUTE */
	0x1B, /* ESCAPE */
	0x1C, /* FILE SEPARATOR */
	0x1D, /* GROUP SEPARATOR */
	0x1E, /* RECORD SEPARATOR */
	0x1F, /* UNIT SEPARATOR */

	0x20, /* SPACE */
	0x21, /* EXCLAMATION MARK */
	0x22, /* QUOTATION MARK */
	0x23, /* NUMBER SIGN */
	0x24, /* DOLLAR SIGN */
	0x25, /* PERCENT SIGN */
	0x26, /* AMPERSAND */
	0x27, /* APOSTROPHE */
	0x28, /* LEFT PARENTHESIS */
	0x29, /* RIGHT PARENTHESIS */
	0x2A, /* ASTERISK */
	0x2B, /* PLUS SIGN */
	0x2C, /* COMMA */
	0x2D, /* HYPHEN-MINUS */
	0x2E, /* FULL STOP */
	0x2F, /* SOLIDUS */
	0x30, /* DIGIT ZERO */
	0x31, /* DIGIT ONE */
	0x32, /* DIGIT TWO */
	0x33, /* DIGIT THREE */
	0x34, /* DIGIT FOUR */
	0x35, /* DIGIT FIVE */
	0x36, /* DIGIT SIX */
	0x37, /* DIGIT SEVEN */
	0x38, /* DIGIT EIGHT */
	0x39, /* DIGIT NINE */
	0x3A, /* COLON */
	0x3B, /* SEMICOLON */
	0x3C, /* LESS-THAN SIGN */
	0x3D, /* EQUALS SIGN */
	0x3E, /* GREATER-THAN SIGN */
	0x3F, /* QUESTION MARK */

	0x40, /* COMMERCIAL AT */
	0x61, /* LATIN CAPITAL LETTER A */
	0x62, /* LATIN CAPITAL LETTER B */
	0x63, /* LATIN CAPITAL LETTER C */
	0x64, /* LATIN CAPITAL LETTER D */
	0x65, /* LATIN CAPITAL LETTER E */
	0x66, /* LATIN CAPITAL LETTER F */
	0x67, /* LATIN CAPITAL LETTER G */
	0x68, /* LATIN CAPITAL LETTER H */
	0x69, /* LATIN CAPITAL LETTER I */
	0x6A, /* LATIN CAPITAL LETTER J */
	0x6B, /* LATIN CAPITAL LETTER K */
	0x6C, /* LATIN CAPITAL LETTER L */
	0x6D, /* LATIN CAPITAL LETTER M */
	0x6E, /* LATIN CAPITAL LETTER N */
	0x6F, /* LATIN CAPITAL LETTER O */
	0x70, /* LATIN CAPITAL LETTER P */
	0x71, /* LATIN CAPITAL LETTER Q */
	0x72, /* LATIN CAPITAL LETTER R */
	0x73, /* LATIN CAPITAL LETTER S */
	0x74, /* LATIN CAPITAL LETTER T */
	0x75, /* LATIN CAPITAL LETTER U */
	0x76, /* LATIN CAPITAL LETTER V */
	0x77, /* LATIN CAPITAL LETTER W */
	0x78, /* LATIN CAPITAL LETTER X */
	0x79, /* LATIN CAPITAL LETTER Y */
	0x7A, /* LATIN CAPITAL LETTER Z */
	0x5B, /* LEFT SQUARE BRACKET */
	0x5C, /* REVERSE SOLIDUS */
	0x5D, /* RIGHT SQUARE BRACKET */
	0x5E, /* CIRCUMFLEX ACCENT */
	0x5F, /* LOW LINE */

	0x60, /* GRAVE ACCENT */
	0x61, /* LATIN SMALL LETTER A */
	0x62, /* LATIN SMALL LETTER B */
	0x63, /* LATIN SMALL LETTER C */
	0x64, /* LATIN SMALL LETTER D */
	0x65, /* LATIN SMALL LETTER E */
	0x66, /* LATIN SMALL LETTER F */
	0x67, /* LATIN SMALL LETTER G */
	0x68, /* LATIN SMALL LETTER H */
	0x69, /* LATIN SMALL LETTER I */
	0x6A, /* LATIN SMALL LETTER J */
	0x6B, /* LATIN SMALL LETTER K */
	0x6C, /* LATIN SMALL LETTER L */
	0x6D, /* LATIN SMALL LETTER M */
	0x6E, /* LATIN SMALL LETTER N */
	0x6F, /* LATIN SMALL LETTER O */
	0x70, /* LATIN SMALL LETTER P */
	0x71, /* LATIN SMALL LETTER Q */
	0x72, /* LATIN SMALL LETTER R */
	0x73, /* LATIN SMALL LETTER S */
	0x74, /* LATIN SMALL LETTER T */
	0x75, /* LATIN SMALL LETTER U */
	0x76, /* LATIN SMALL LETTER V */
	0x77, /* LATIN SMALL LETTER W */
	0x78, /* LATIN SMALL LETTER X */
	0x79, /* LATIN SMALL LETTER Y */
	0x7A, /* LATIN SMALL LETTER Z */
	0x7B, /* LEFT CURLY BRACKET */
	0x7C, /* VERTICAL LINE */
	0x7D, /* RIGHT CURLY BRACKET */
	0x7E, /* TILDE */
	0x7F, /* DELETE */

	0x80, /* <control> */
	0x81, /* <control> */
	0x82, /* <control> */
	0x83, /* <control> */
	0x84, /* <control> */
	0x85, /* <control> */
	0x86, /* <control> */
	0x87, /* <control> */
	0x88, /* <control> */
	0x89, /* <control> */
	0x8A, /* <control> */
	0x8B, /* <control> */
	0x8C, /* <control> */
	0x8D, /* <control> */
	0x8E, /* <control> */
	0x8F, /* <control> */
	0x90, /* <control> */
	0x91, /* <control> */
	0x92, /* <control> */
	0x93, /* <control> */
	0x94, /* <control> */
	0x95, /* <control> */
	0x96, /* <control> */
	0x97, /* <control> */
	0x98, /* <control> */
	0x99, /* <control> */
	0x9A, /* <control> */
	0x9B, /* <control> */
	0x9C, /* <control> */
	0x9D, /* <control> */
	0x9E, /* <control> */
	0x9F, /* <control> */

	0xA0	,	/*	NO-BREAK SPACE */
	0xA1	,	/*	LEFT SINGLE QUOTATION MARK */
	0xA2,	/*	RIGHT SINGLE QUOTATION MARK */
	0xA3	,	/*	POUND SIGN */
	0xA4	,	/*	EURO SIGN */
	0xA5,	/*	DRACHMA SIGN */
	0xA6	,	/*	BROKEN BAR */
	0xA7	,	/*	SECTION SIGN */
	0xA8	,	/*	DIAERESIS */
	0xA9	,	/*	COPYRIGHT SIGN */
	0xAA	,	/*	GREEK YPOGEGRAMMENI */
	0xAB,	/*	LEFT-POINTING DOUBLE ANGLE QUOTATION MARK */
	0xAC,	/*	NOT SIGN */
	0xAD,	/*	SOFT HYPHEN */
	0xAF	,	/*	HORIZONTAL BAR */
	0xB0,	/*	DEGREE SIGN */
	0xB1,	/*	PLUS-MINUS SIGN */
	0xB2	,	/*	SUPERSCRIPT TWO */
	0xB3	,	/*	SUPERSCRIPT THREE */
	0xB4,	/*	GREEK TONOS */
	0xB5,	/*	GREEK DIALYTIKA TONOS */
	0xDC,	/*	GREEK CAPITAL LETTER ALPHA WITH TONOS */
	0xB7,	/*	MIDDLE DOT */
	0xDD,	/*	GREEK CAPITAL LETTER EPSILON WITH TONOS */
	0xDE,	/*	GREEK CAPITAL LETTER ETA WITH TONOS */
	0xDF	,	/*	GREEK CAPITAL LETTER IOTA WITH TONOS */
	0xBB,	/*	RIGHT-POINTING DOUBLE ANGLE QUOTATION MARK */
	0xFC,	/*	GREEK CAPITAL LETTER OMICRON WITH TONOS */
	0xBD,	/*	VULGAR FRACTION ONE HALF */
	0xFD,	/*	GREEK CAPITAL LETTER UPSILON WITH TONOS */
	0xFE,	/*	GREEK CAPITAL LETTER OMEGA WITH TONOS */

	0xC0,	/*	GREEK SMALL LETTER IOTA WITH DIALYTIKA AND TONOS */
	0xE1,	/*	GREEK CAPITAL LETTER ALPHA */
	0xE2,	/*	GREEK CAPITAL LETTER BETA */
	0xE3,	/*	GREEK CAPITAL LETTER GAMMA */
	0xE4,	/*	GREEK CAPITAL LETTER DELTA */
	0xE5,	/*	GREEK CAPITAL LETTER EPSILON */
	0xE6,	/*	GREEK CAPITAL LETTER ZETA */
	0xE7,	/*	GREEK CAPITAL LETTER ETA */
	0xE8,	/*	GREEK CAPITAL LETTER THETA */
	0xE9,	/*	GREEK CAPITAL LETTER IOTA */
	0xEA,	/*	GREEK CAPITAL LETTER KAPPA */
	0xEB,	/*	GREEK CAPITAL LETTER LAMDA */
	0xEC,	/*	GREEK CAPITAL LETTER MU */
	0xED,	/*	GREEK CAPITAL LETTER NU */
	0xEE,	/*	GREEK CAPITAL LETTER XI */
	0xEF	,	/*	GREEK CAPITAL LETTER OMICRON */
	0xF0,	/*	GREEK CAPITAL LETTER PI */
	0xF1,	/*	GREEK CAPITAL LETTER RHO */
	0xF3,	/*	GREEK CAPITAL LETTER SIGMA */
	0xF4,	/*	GREEK CAPITAL LETTER TAU */
	0xF5,	/*	GREEK CAPITAL LETTER UPSILON */
	0xF6,	/*	GREEK CAPITAL LETTER PHI */
	0xF7,	/*	GREEK CAPITAL LETTER CHI */
	0xF8,	/*	GREEK CAPITAL LETTER PSI */
	0xF9,	/*	GREEK CAPITAL LETTER OMEGA */
	0xDA,	/*	GREEK CAPITAL LETTER IOTA WITH DIALYTIKA */
	0xDB,	/*	GREEK CAPITAL LETTER UPSILON WITH DIALYTIKA */
	0xDC,	/*	GREEK SMALL LETTER ALPHA WITH TONOS */
	0xDD,	/*	GREEK SMALL LETTER EPSILON WITH TONOS */
	0xDE,	/*	GREEK SMALL LETTER ETA WITH TONOS */
	0xDF,	/*	GREEK SMALL LETTER IOTA WITH TONOS */

	0xE0	,	/*	GREEK SMALL LETTER UPSILON WITH DIALYTIKA AND TONOS */
	0xE1,	/*	GREEK SMALL LETTER ALPHA */
	0xE2	,	/*	GREEK SMALL LETTER BETA */
	0xE3,	/*	GREEK SMALL LETTER GAMMA */
	0xE4	,	/*	GREEK SMALL LETTER DELTA */
	0xE5,	/*	GREEK SMALL LETTER EPSILON */
	0xE6	,	/*	GREEK SMALL LETTER ZETA */
	0xE7,	/*	GREEK SMALL LETTER ETA */
	0xE8,	/*	GREEK SMALL LETTER THETA */
	0xE9,	/*	GREEK SMALL LETTER IOTA */
	0xEA	,	/*	GREEK SMALL LETTER KAPPA */
	0xEB	,	/*	GREEK SMALL LETTER LAMDA */
	0xEC,	/*	GREEK SMALL LETTER MU */
	0xED,	/*	GREEK SMALL LETTER NU */
	0xEE	,	/*	GREEK SMALL LETTER XI */
	0xEF	,	/*	GREEK SMALL LETTER OMICRON */
	0xF0,	/*	GREEK SMALL LETTER PI */
	0xF1,	/*	GREEK SMALL LETTER RHO */
	0xF2	,	/*	GREEK SMALL LETTER FINAL SIGMA */
	0xF3	,	/*	GREEK SMALL LETTER SIGMA */
	0xF4,	/*	GREEK SMALL LETTER TAU */
	0xF5	,	/*	GREEK SMALL LETTER UPSILON */
	0xF6,	/*	GREEK SMALL LETTER PHI */
	0xF7	,	/*	GREEK SMALL LETTER CHI */
	0xF8	,	/*	GREEK SMALL LETTER PSI */
	0xF9	,	/*	GREEK SMALL LETTER OMEGA */
	0xFA	,	/*	GREEK SMALL LETTER IOTA WITH DIALYTIKA */
	0xFB,	/*	GREEK SMALL LETTER UPSILON WITH DIALYTIKA */
	0xFC	,	/*	GREEK SMALL LETTER OMICRON WITH TONOS */
	0xFD,	/*	GREEK SMALL LETTER UPSILON WITH TONOS */
	0xFE	,	/*	GREEK SMALL LETTER OMEGA WITH TONOS */
	0xFF  /* DOT ABOVE */
};

static const UBYTE LangConvToUpper[256] =
{
	0x00, /* NULL */
	0x01, /* START OF HEADING */
	0x02, /* START OF TEXT */
	0x03, /* END OF TEXT */
	0x04, /* END OF TRANSMISSION */
	0x05, /* ENQUIRY */
	0x06, /* ACKNOWLEDGE */
	0x07, /* BELL */
	0x08, /* BACKSPACE */
	0x09, /* HORIZONTAL TABULATION */
	0x0A, /* LINE FEED */
	0x0B, /* VERTICAL TABULATION */
	0x0C, /* FORM FEED */
	0x0D, /* CARRIAGE RETURN */
	0x0E, /* SHIFT OUT */
	0x0F, /* SHIFT IN */
	0x10, /* DATA LINK ESCAPE */
	0x11, /* DEVICE CONTROL ONE */
	0x12, /* DEVICE CONTROL TWO */
	0x13, /* DEVICE CONTROL THREE */
	0x14, /* DEVICE CONTROL FOUR */
	0x15, /* NEGATIVE ACKNOWLEDGE */
	0x16, /* SYNCHRONOUS IDLE */
	0x17, /* END OF TRANSMISSION BLOCK */
	0x18, /* CANCEL */
	0x19, /* END OF MEDIUM */
	0x1A, /* SUBSTITUTE */
	0x1B, /* ESCAPE */
	0x1C, /* FILE SEPARATOR */
	0x1D, /* GROUP SEPARATOR */
	0x1E, /* RECORD SEPARATOR */
	0x1F, /* UNIT SEPARATOR */

	0x20, /* SPACE */
	0x21, /* EXCLAMATION MARK */
	0x22, /* QUOTATION MARK */
	0x23, /* NUMBER SIGN */
	0x24, /* DOLLAR SIGN */
	0x25, /* PERCENT SIGN */
	0x26, /* AMPERSAND */
	0x27, /* APOSTROPHE */
	0x28, /* LEFT PARENTHESIS */
	0x29, /* RIGHT PARENTHESIS */
	0x2A, /* ASTERISK */
	0x2B, /* PLUS SIGN */
	0x2C, /* COMMA */
	0x2D, /* HYPHEN-MINUS */
	0x2E, /* FULL STOP */
	0x2F, /* SOLIDUS */
	0x30, /* DIGIT ZERO */
	0x31, /* DIGIT ONE */
	0x32, /* DIGIT TWO */
	0x33, /* DIGIT THREE */
	0x34, /* DIGIT FOUR */
	0x35, /* DIGIT FIVE */
	0x36, /* DIGIT SIX */
	0x37, /* DIGIT SEVEN */
	0x38, /* DIGIT EIGHT */
	0x39, /* DIGIT NINE */
	0x3A, /* COLON */
	0x3B, /* SEMICOLON */
	0x3C, /* LESS-THAN SIGN */
	0x3D, /* EQUALS SIGN */
	0x3E, /* GREATER-THAN SIGN */
	0x3F, /* QUESTION MARK */

	0x40, /* COMMERCIAL AT */
	0x41, /* LATIN CAPITAL LETTER A */
	0x42, /* LATIN CAPITAL LETTER B */
	0x43, /* LATIN CAPITAL LETTER C */
	0x44, /* LATIN CAPITAL LETTER D */
	0x45, /* LATIN CAPITAL LETTER E */
	0x46, /* LATIN CAPITAL LETTER F */
	0x47, /* LATIN CAPITAL LETTER G */
	0x48, /* LATIN CAPITAL LETTER H */
	0x49, /* LATIN CAPITAL LETTER I */
	0x4A, /* LATIN CAPITAL LETTER J */
	0x4B, /* LATIN CAPITAL LETTER K */
	0x4C, /* LATIN CAPITAL LETTER L */
	0x4D, /* LATIN CAPITAL LETTER M */
	0x4E, /* LATIN CAPITAL LETTER N */
	0x4F, /* LATIN CAPITAL LETTER O */
	0x50, /* LATIN CAPITAL LETTER P */
	0x51, /* LATIN CAPITAL LETTER Q */
	0x52, /* LATIN CAPITAL LETTER R */
	0x53, /* LATIN CAPITAL LETTER S */
	0x54, /* LATIN CAPITAL LETTER T */
	0x55, /* LATIN CAPITAL LETTER U */
	0x56, /* LATIN CAPITAL LETTER V */
	0x57, /* LATIN CAPITAL LETTER W */
	0x58, /* LATIN CAPITAL LETTER X */
	0x59, /* LATIN CAPITAL LETTER Y */
	0x5A, /* LATIN CAPITAL LETTER Z */
	0x5B, /* LEFT SQUARE BRACKET */
	0x5C, /* REVERSE SOLIDUS */
	0x5D, /* RIGHT SQUARE BRACKET */
	0x5E, /* CIRCUMFLEX ACCENT */
	0x5F, /* LOW LINE */

	0x60, /* GRAVE ACCENT */
	0x41, /* LATIN SMALL LETTER A */
	0x42, /* LATIN SMALL LETTER B */
	0x43, /* LATIN SMALL LETTER C */
	0x44, /* LATIN SMALL LETTER D */
	0x45, /* LATIN SMALL LETTER E */
	0x46, /* LATIN SMALL LETTER F */
	0x47, /* LATIN SMALL LETTER G */
	0x48, /* LATIN SMALL LETTER H */
	0x49, /* LATIN SMALL LETTER I */
	0x4A, /* LATIN SMALL LETTER J */
	0x4B, /* LATIN SMALL LETTER K */
	0x4C, /* LATIN SMALL LETTER L */
	0x4D, /* LATIN SMALL LETTER M */
	0x4E, /* LATIN SMALL LETTER N */
	0x4F, /* LATIN SMALL LETTER O */
	0x50, /* LATIN SMALL LETTER P */
	0x51, /* LATIN SMALL LETTER Q */
	0x52, /* LATIN SMALL LETTER R */
	0x53, /* LATIN SMALL LETTER S */
	0x54, /* LATIN SMALL LETTER T */
	0x55, /* LATIN SMALL LETTER U */
	0x56, /* LATIN SMALL LETTER V */
	0x57, /* LATIN SMALL LETTER W */
	0x58, /* LATIN SMALL LETTER X */
	0x59, /* LATIN SMALL LETTER Y */
	0x5A, /* LATIN SMALL LETTER Z */
	0x7B, /* LEFT CURLY BRACKET */
	0x7C, /* VERTICAL LINE */
	0x7D, /* RIGHT CURLY BRACKET */
	0x7E, /* TILDE */
	0x7F, /* DELETE */

	0x80, /* <control> */
	0x81, /* <control> */
	0x82, /* <control> */
	0x83, /* <control> */
	0x84, /* <control> */
	0x85, /* <control> */
	0x86, /* <control> */
	0x87, /* <control> */
	0x88, /* <control> */
	0x89, /* <control> */
	0x8A, /* <control> */
	0x8B, /* <control> */
	0x8C, /* <control> */
	0x8D, /* <control> */
	0x8E, /* <control> */
	0x8F, /* <control> */
	0x90, /* <control> */
	0x91, /* <control> */
	0x92, /* <control> */
	0x93, /* <control> */
	0x94, /* <control> */
	0x95, /* <control> */
	0x96, /* <control> */
	0x97, /* <control> */
	0x98, /* <control> */
	0x99, /* <control> */
	0x9A, /* <control> */
	0x9B, /* <control> */
	0x9C, /* <control> */
	0x9D, /* <control> */
	0x9E, /* <control> */
	0x9F, /* <control> */

	0xA0	,	/*	NO-BREAK SPACE */
	0xA1	,	/*	LEFT SINGLE QUOTATION MARK */
	0xA2,	/*	RIGHT SINGLE QUOTATION MARK */
	0xA3	,	/*	POUND SIGN */
	0xA4	,	/*	EURO SIGN */
	0xA5,	/*	DRACHMA SIGN */
	0xA6	,	/*	BROKEN BAR */
	0xA7	,	/*	SECTION SIGN */
	0xA8	,	/*	DIAERESIS */
	0xA9	,	/*	COPYRIGHT SIGN */
	0xAA	,	/*	GREEK YPOGEGRAMMENI */
	0xAB,	/*	LEFT-POINTING DOUBLE ANGLE QUOTATION MARK */
	0xAC,	/*	NOT SIGN */
	0xAD,	/*	SOFT HYPHEN */
	0xAF	,	/*	HORIZONTAL BAR */
	0xB0,	/*	DEGREE SIGN */
	0xB1,	/*	PLUS-MINUS SIGN */
	0xB2	,	/*	SUPERSCRIPT TWO */
	0xB3	,	/*	SUPERSCRIPT THREE */
	0xB4,	/*	GREEK TONOS */
	0xB5,	/*	GREEK DIALYTIKA TONOS */
	0xB6	,	/*	GREEK CAPITAL LETTER ALPHA WITH TONOS */
	0xB7,	/*	MIDDLE DOT */
	0xB8	,	/*	GREEK CAPITAL LETTER EPSILON WITH TONOS */
	0xB9,	/*	GREEK CAPITAL LETTER ETA WITH TONOS */
	0xBA	,	/*	GREEK CAPITAL LETTER IOTA WITH TONOS */
	0xBB,	/*	RIGHT-POINTING DOUBLE ANGLE QUOTATION MARK */
	0xBC,	/*	GREEK CAPITAL LETTER OMICRON WITH TONOS */
	0xBD,	/*	VULGAR FRACTION ONE HALF */
	0xBE	,	/*	GREEK CAPITAL LETTER UPSILON WITH TONOS */
	0xBF	,	/*	GREEK CAPITAL LETTER OMEGA WITH TONOS */

	0xC0,	/*	GREEK SMALL LETTER IOTA WITH DIALYTIKA AND TONOS */
	0xC1,	/*	GREEK CAPITAL LETTER ALPHA */
	0xC2,	/*	GREEK CAPITAL LETTER BETA */
	0xC3,	/*	GREEK CAPITAL LETTER GAMMA */
	0xC4,	/*	GREEK CAPITAL LETTER DELTA */
	0xC5,	/*	GREEK CAPITAL LETTER EPSILON */
	0xC6,	/*	GREEK CAPITAL LETTER ZETA */
	0xC7,	/*	GREEK CAPITAL LETTER ETA */
	0xC8,	/*	GREEK CAPITAL LETTER THETA */
	0xC9,	/*	GREEK CAPITAL LETTER IOTA */
	0xCA,	/*	GREEK CAPITAL LETTER KAPPA */
	0xCB,	/*	GREEK CAPITAL LETTER LAMDA */
	0xCC,	/*	GREEK CAPITAL LETTER MU */
	0xCD,	/*	GREEK CAPITAL LETTER NU */
	0xCE,	/*	GREEK CAPITAL LETTER XI */
	0xCF	,	/*	GREEK CAPITAL LETTER OMICRON */
	0xD0,	/*	GREEK CAPITAL LETTER PI */
	0xD1,	/*	GREEK CAPITAL LETTER RHO */
	0xD3,	/*	GREEK CAPITAL LETTER SIGMA */
	0xD4,	/*	GREEK CAPITAL LETTER TAU */
	0xD5,	/*	GREEK CAPITAL LETTER UPSILON */
	0xD6,	/*	GREEK CAPITAL LETTER PHI */
	0xD7,	/*	GREEK CAPITAL LETTER CHI */
	0xD8,	/*	GREEK CAPITAL LETTER PSI */
	0xD9,	/*	GREEK CAPITAL LETTER OMEGA */
	0xDA,	/*	GREEK CAPITAL LETTER IOTA WITH DIALYTIKA */
	0xDB,	/*	GREEK CAPITAL LETTER UPSILON WITH DIALYTIKA */
	0xB6,	/*	GREEK SMALL LETTER ALPHA WITH TONOS */
	0xB8,	/*	GREEK SMALL LETTER EPSILON WITH TONOS */
	0xB9,	/*	GREEK SMALL LETTER ETA WITH TONOS */
	0xBA,	/*	GREEK SMALL LETTER IOTA WITH TONOS */

	0xBE	,	/*	GREEK SMALL LETTER UPSILON WITH DIALYTIKA AND TONOS */
	0xC1,	/*	GREEK SMALL LETTER ALPHA */
	0xC2,	/*	GREEK SMALL LETTER BETA */
	0xC3,	/*	GREEK SMALL LETTER GAMMA */
	0xC4,	/*	GREEK SMALL LETTER DELTA */
	0xC5,	/*	GREEK SMALL LETTER EPSILON */
	0xC6,	/*	GREEK SMALL LETTER ZETA */
	0xC7,	/*	GREEK SMALL LETTER ETA */
	0xC8,	/*	GREEK SMALL LETTER THETA */
	0xC9,	/*	GREEK SMALL LETTER IOTA */
	0xCA,	/*	GREEK SMALL LETTER KAPPA */
	0xCB,	/*	GREEK SMALL LETTER LAMDA */
	0xCC,	/*	GREEK SMALL LETTER MU */
	0xCD,	/*	GREEK SMALL LETTER NU */
	0xCE,	/*	GREEK SMALL LETTER XI */
	0xCF	,	/*	GREEK SMALL LETTER OMICRON */
	0xD0,	/*	GREEK SMALL LETTER PI */
	0xD1,	/*	GREEK SMALL LETTER RHO */
	0xD3,	/*	GREEK SMALL LETTER FINAL SIGMA */
	0xD3,	/*	GREEK SMALL LETTER SIGMA */
	0xD4,	/*	GREEK SMALL LETTER TAU */
	0xD5,	/*	GREEK SMALL LETTER UPSILON */
	0xD6,	/*	GREEK SMALL LETTER PHI */
	0xD7,	/*	GREEK SMALL LETTER CHI */
	0xD8,	/*	GREEK SMALL LETTER PSI */
	0xD9,	/*	GREEK SMALL LETTER OMEGA */
	0xC9,	/*	GREEK SMALL LETTER IOTA WITH DIALYTIKA */
	0xD5,	/*	GREEK SMALL LETTER UPSILON WITH DIALYTIKA */
	0xCF	,	/*	GREEK SMALL LETTER OMICRON WITH TONOS */
	0xD5,	/*	GREEK SMALL LETTER UPSILON WITH TONOS */
	0xD9,	/*	GREEK SMALL LETTER OMEGA WITH TONOS */
	0xFF  /* DOT ABOVE */
};

// This table SHOULD BE REVISED before using for other languages using
// ISO-8859-7 encoding. I assumed here all accented character have ordinals
// equal to non-accented version, it is true for Polish but need not to be
// true for other languages. [krashan]

static const UBYTE GreekCollate1[256] =
{
	0x00, /* NULL */
	0x01, /* START OF HEADING */
	0x02, /* START OF TEXT */
	0x03, /* END OF TEXT */
	0x04, /* END OF TRANSMISSION */
	0x05, /* ENQUIRY */
	0x06, /* ACKNOWLEDGE */
	0x07, /* BELL */
	0x08, /* BACKSPACE */
	0x09, /* HORIZONTAL TABULATION */
	0x0A, /* LINE FEED */
	0x0B, /* VERTICAL TABULATION */
	0x0C, /* FORM FEED */
	0x0D, /* CARRIAGE RETURN */
	0x0E, /* SHIFT OUT */
	0x0F, /* SHIFT IN */
	0x10, /* DATA LINK ESCAPE */
	0x11, /* DEVICE CONTROL ONE */
	0x12, /* DEVICE CONTROL TWO */
	0x13, /* DEVICE CONTROL THREE */
	0x14, /* DEVICE CONTROL FOUR */
	0x15, /* NEGATIVE ACKNOWLEDGE */
	0x16, /* SYNCHRONOUS IDLE */
	0x17, /* END OF TRANSMISSION BLOCK */
	0x18, /* CANCEL */
	0x19, /* END OF MEDIUM */
	0x1A, /* SUBSTITUTE */
	0x1B, /* ESCAPE */
	0x1C, /* FILE SEPARATOR */
	0x1D, /* GROUP SEPARATOR */
	0x1E, /* RECORD SEPARATOR */
	0x1F, /* UNIT SEPARATOR */

	0x20, /* SPACE */
	0x21, /* EXCLAMATION MARK */
	0x22, /* QUOTATION MARK */
	0x23, /* NUMBER SIGN */
	0x24, /* DOLLAR SIGN */
	0x25, /* PERCENT SIGN */
	0x26, /* AMPERSAND */
	0x27, /* APOSTROPHE */
	0x28, /* LEFT PARENTHESIS */
	0x29, /* RIGHT PARENTHESIS */
	0x2A, /* ASTERISK */
	0x2B, /* PLUS SIGN */
	0x2C, /* COMMA */
	0x2D, /* HYPHEN-MINUS */
	0x2E, /* FULL STOP */
	0x2F, /* SOLIDUS */
	0x30, /* DIGIT ZERO */
	0x31, /* DIGIT ONE */
	0x32, /* DIGIT TWO */
	0x33, /* DIGIT THREE */
	0x34, /* DIGIT FOUR */
	0x35, /* DIGIT FIVE */
	0x36, /* DIGIT SIX */
	0x37, /* DIGIT SEVEN */
	0x38, /* DIGIT EIGHT */
	0x39, /* DIGIT NINE */
	0x3A, /* COLON */
	0x3B, /* SEMICOLON */
	0x3C, /* LESS-THAN SIGN */
	0x3D, /* EQUALS SIGN */
	0x3E, /* GREATER-THAN SIGN */
	0x3F, /* QUESTION MARK */

	0x40, /* COMMERCIAL AT */
	0x41, /* LATIN CAPITAL LETTER A */
	0x42, /* LATIN CAPITAL LETTER B */
	0x43, /* LATIN CAPITAL LETTER C */
	0x44, /* LATIN CAPITAL LETTER D */
	0x45, /* LATIN CAPITAL LETTER E */
	0x46, /* LATIN CAPITAL LETTER F */
	0x47, /* LATIN CAPITAL LETTER G */
	0x48, /* LATIN CAPITAL LETTER H */
	0x49, /* LATIN CAPITAL LETTER I */
	0x4A, /* LATIN CAPITAL LETTER J */
	0x4B, /* LATIN CAPITAL LETTER K */
	0x4C, /* LATIN CAPITAL LETTER L */
	0x4D, /* LATIN CAPITAL LETTER M */
	0x4E, /* LATIN CAPITAL LETTER N */
	0x4F, /* LATIN CAPITAL LETTER O */
	0x50, /* LATIN CAPITAL LETTER P */
	0x51, /* LATIN CAPITAL LETTER Q */
	0x52, /* LATIN CAPITAL LETTER R */
	0x53, /* LATIN CAPITAL LETTER S */
	0x54, /* LATIN CAPITAL LETTER T */
	0x55, /* LATIN CAPITAL LETTER U */
	0x56, /* LATIN CAPITAL LETTER V */
	0x57, /* LATIN CAPITAL LETTER W */
	0x58, /* LATIN CAPITAL LETTER X */
	0x59, /* LATIN CAPITAL LETTER Y */
	0x5A, /* LATIN CAPITAL LETTER Z */
	0x5B, /* LEFT SQUARE BRACKET */
	0x5C, /* REVERSE SOLIDUS */
	0x5D, /* RIGHT SQUARE BRACKET */
	0x5E, /* CIRCUMFLEX ACCENT */
	0x5F, /* LOW LINE */

	0x60, /* GRAVE ACCENT */
	0x41, /* LATIN SMALL LETTER A */
	0x42, /* LATIN SMALL LETTER B */
	0x43, /* LATIN SMALL LETTER C */
	0x44, /* LATIN SMALL LETTER D */
	0x45, /* LATIN SMALL LETTER E */
	0x46, /* LATIN SMALL LETTER F */
	0x47, /* LATIN SMALL LETTER G */
	0x48, /* LATIN SMALL LETTER H */
	0x49, /* LATIN SMALL LETTER I */
	0x4A, /* LATIN SMALL LETTER J */
	0x4B, /* LATIN SMALL LETTER K */
	0x4C, /* LATIN SMALL LETTER L */
	0x4D, /* LATIN SMALL LETTER M */
	0x4E, /* LATIN SMALL LETTER N */
	0x4F, /* LATIN SMALL LETTER O */
	0x50, /* LATIN SMALL LETTER P */
	0x51, /* LATIN SMALL LETTER Q */
	0x52, /* LATIN SMALL LETTER R */
	0x53, /* LATIN SMALL LETTER S */
	0x54, /* LATIN SMALL LETTER T */
	0x55, /* LATIN SMALL LETTER U */
	0x56, /* LATIN SMALL LETTER V */
	0x57, /* LATIN SMALL LETTER W */
	0x58, /* LATIN SMALL LETTER X */
	0x59, /* LATIN SMALL LETTER Y */
	0x5A, /* LATIN SMALL LETTER Z */
	0x61, /* LEFT CURLY BRACKET */
	0x62, /* VERTICAL LINE */
	0x63, /* RIGHT CURLY BRACKET */
	0x64, /* TILDE */
	0x65, /* DELETE */

	0x80, /* <control> */
	0x81, /* <control> */
	0x82, /* <control> */
	0x83, /* <control> */
	0x84, /* <control> */
	0x85, /* <control> */
	0x86, /* <control> */
	0x87, /* <control> */
	0x88, /* <control> */
	0x89, /* <control> */
	0x8A, /* <control> */
	0x8B, /* <control> */
	0x8C, /* <control> */
	0x8D, /* <control> */
	0x8E, /* <control> */
	0x8F, /* <control> */
	0x90, /* <control> */
	0x91, /* <control> */
	0x92, /* <control> */
	0x93, /* <control> */
	0x94, /* <control> */
	0x95, /* <control> */
	0x96, /* <control> */
	0x97, /* <control> */
	0x98, /* <control> */
	0x99, /* <control> */
	0x9A, /* <control> */
	0x9B, /* <control> */
	0x9C, /* <control> */
	0x9D, /* <control> */
	0x9E, /* <control> */
	0x9F, /* <control> */

	0xA0	,	/*	NO-BREAK SPACE */
	0xA1	,	/*	LEFT SINGLE QUOTATION MARK */
	0xA2,	/*	RIGHT SINGLE QUOTATION MARK */
	0xA3	,	/*	POUND SIGN */
	0xA4	,	/*	EURO SIGN */
	0xA5,	/*	DRACHMA SIGN */
	0xA6	,	/*	BROKEN BAR */
	0xA7	,	/*	SECTION SIGN */
	0xA8	,	/*	DIAERESIS */
	0xA9	,	/*	COPYRIGHT SIGN */
	0xAA	,	/*	GREEK YPOGEGRAMMENI */
	0xAB,	/*	LEFT-POINTING DOUBLE ANGLE QUOTATION MARK */
	0xAC,	/*	NOT SIGN */
	0xAD,	/*	SOFT HYPHEN */
	0xAF	,	/*	HORIZONTAL BAR */
	0xB0,	/*	DEGREE SIGN */
	0xB1,	/*	PLUS-MINUS SIGN */
	0xB2	,	/*	SUPERSCRIPT TWO */
	0xB3	,	/*	SUPERSCRIPT THREE */
	0xB4,	/*	GREEK TONOS */
	0xB5,	/*	GREEK DIALYTIKA TONOS */
	0xB7,	/*	MIDDLE DOT */
	0xBB,	/*	RIGHT-POINTING DOUBLE ANGLE QUOTATION MARK */
	0xBD,	/*	VULGAR FRACTION ONE HALF */

	0xC1,	/*	GREEK CAPITAL LETTER ALPHA */
	0xC1	,	/*	GREEK CAPITAL LETTER ALPHA WITH TONOS */
	0xC2,	/*	GREEK CAPITAL LETTER BETA */
	0xC3,	/*	GREEK CAPITAL LETTER GAMMA */
	0xC4,	/*	GREEK CAPITAL LETTER DELTA */
	0xC5,	/*	GREEK CAPITAL LETTER EPSILON */
	0xC5,	/*	GREEK CAPITAL LETTER EPSILON WITH TONOS */
	0xC6,	/*	GREEK CAPITAL LETTER ZETA */
	0xC7,	/*	GREEK CAPITAL LETTER ETA */
	0xC7,	/*	GREEK CAPITAL LETTER ETA WITH TONOS */
	0xC8,	/*	GREEK CAPITAL LETTER THETA */
	0xC9,	/*	GREEK CAPITAL LETTER IOTA */
	0xC9	,	/*	GREEK CAPITAL LETTER IOTA WITH TONOS */
	0xC9,	/*	GREEK CAPITAL LETTER IOTA WITH DIALYTIKA */
	0xCA,	/*	GREEK CAPITAL LETTER KAPPA */
	0xCB,	/*	GREEK CAPITAL LETTER LAMDA */
	0xCC,	/*	GREEK CAPITAL LETTER MU */
	0xCD,	/*	GREEK CAPITAL LETTER NU */
	0xCE,	/*	GREEK CAPITAL LETTER XI */
	0xCF	,	/*	GREEK CAPITAL LETTER OMICRON */
	0xCF,	/*	GREEK CAPITAL LETTER OMICRON WITH TONOS */
	0xD0,	/*	GREEK CAPITAL LETTER PI */
	0xD1,	/*	GREEK CAPITAL LETTER RHO */
	0xD3,	/*	GREEK CAPITAL LETTER SIGMA */
	0xD4,	/*	GREEK CAPITAL LETTER TAU */
	0xD5,	/*	GREEK CAPITAL LETTER UPSILON */
	0xD5,	/*	GREEK CAPITAL LETTER UPSILON WITH TONOS */
	0xD5,	/*	GREEK CAPITAL LETTER UPSILON WITH DIALYTIKA */
	0xD6,	/*	GREEK CAPITAL LETTER PHI */
	0xD7,	/*	GREEK CAPITAL LETTER CHI */
	0xD8,	/*	GREEK CAPITAL LETTER PSI */
	0xD9,	/*	GREEK CAPITAL LETTER OMEGA */
	0xD9,	/*	GREEK CAPITAL LETTER OMEGA WITH TONOS */

	0xE1,	/*	GREEK SMALL LETTER ALPHA */
	0xE1,	/*	GREEK SMALL LETTER ALPHA WITH TONOS */
	0xE2	,	/*	GREEK SMALL LETTER BETA */
	0xE3,	/*	GREEK SMALL LETTER GAMMA */
	0xE4	,	/*	GREEK SMALL LETTER DELTA */
	0xE5,	/*	GREEK SMALL LETTER EPSILON */
	0xE5,	/*	GREEK SMALL LETTER EPSILON WITH TONOS */
	0xE6	,	/*	GREEK SMALL LETTER ZETA */
	0xE7,	/*	GREEK SMALL LETTER ETA */
	0xE7,	/*	GREEK SMALL LETTER ETA WITH TONOS */
	0xE8,	/*	GREEK SMALL LETTER THETA */
	0xE9,	/*	GREEK SMALL LETTER IOTA */
	0xE9,	/*	GREEK SMALL LETTER IOTA WITH TONOS */
	0xE9,	/*	GREEK SMALL LETTER IOTA WITH DIALYTIKA */
	0xE9,	/*	GREEK SMALL LETTER IOTA WITH DIALYTIKA AND TONOS */
	0xEA	,	/*	GREEK SMALL LETTER KAPPA */
	0xEB	,	/*	GREEK SMALL LETTER LAMDA */
	0xEC,	/*	GREEK SMALL LETTER MU */
	0xED,	/*	GREEK SMALL LETTER NU */
	0xEE	,	/*	GREEK SMALL LETTER XI */
	0xEF	,	/*	GREEK SMALL LETTER OMICRON */
	0xEF,	/*	GREEK SMALL LETTER OMICRON WITH TONOS */
	0xF0,	/*	GREEK SMALL LETTER PI */
	0xF1,	/*	GREEK SMALL LETTER RHO */
	0xF3	,	/*	GREEK SMALL LETTER SIGMA */
	0xF3	,	/*	GREEK SMALL LETTER FINAL SIGMA */
	0xF4,	/*	GREEK SMALL LETTER TAU */
	0xF5	,	/*	GREEK SMALL LETTER UPSILON */
	0xF5,	/*	GREEK SMALL LETTER UPSILON WITH DIALYTIKA */
	0xF5,	/*	GREEK SMALL LETTER UPSILON WITH TONOS */
	0xF5	,	/*	GREEK SMALL LETTER UPSILON WITH DIALYTIKA AND TONOS */
	0xF6,	/*	GREEK SMALL LETTER PHI */
	0xF7	,	/*	GREEK SMALL LETTER CHI */
	0xF8	,	/*	GREEK SMALL LETTER PSI */
	0xF9	,	/*	GREEK SMALL LETTER OMEGA */
	0xF9	,	/*	GREEK SMALL LETTER OMEGA WITH TONOS */
	0xFF  /* DOT ABOVE */
};

///
/// GreekCollate2[]

// This table SHOULD NOT be used without changes in languages using ISO-8859-7
// encoding other than Polish. Collating order for accented characters is
// defined here as following:
//
// - character without accent,
// - Polish specific accented character (for Z order is Z WITH ACUTE, Z WITH DOT
//   ABOVE),
// - character with ring above (if exists),
// - character with acute (if exists and if not Polish specific),
// - character with double acute (if exists),
// - character with circumflex (if exists),
// - character with breve (if exists),
// - character with dieresis (if exists),
// - character with caron (if exists),
// - character with cedilla (if exists).
//
//  This order is probably not right for any language except Polish. [krashan]

static const UBYTE GreekCollate2[256] =
{
	0x00, /* NULL */
	0x01, /* START OF HEADING */
	0x02, /* START OF TEXT */
	0x03, /* END OF TEXT */
	0x04, /* END OF TRANSMISSION */
	0x05, /* ENQUIRY */
	0x06, /* ACKNOWLEDGE */
	0x07, /* BELL */
	0x08, /* BACKSPACE */
	0x09, /* HORIZONTAL TABULATION */
	0x0A, /* LINE FEED */
	0x0B, /* VERTICAL TABULATION */
	0x0C, /* FORM FEED */
	0x0D, /* CARRIAGE RETURN */
	0x0E, /* SHIFT OUT */
	0x0F, /* SHIFT IN */
	0x10, /* DATA LINK ESCAPE */
	0x11, /* DEVICE CONTROL ONE */
	0x12, /* DEVICE CONTROL TWO */
	0x13, /* DEVICE CONTROL THREE */
	0x14, /* DEVICE CONTROL FOUR */
	0x15, /* NEGATIVE ACKNOWLEDGE */
	0x16, /* SYNCHRONOUS IDLE */
	0x17, /* END OF TRANSMISSION BLOCK */
	0x18, /* CANCEL */
	0x19, /* END OF MEDIUM */
	0x1A, /* SUBSTITUTE */
	0x1B, /* ESCAPE */
	0x1C, /* FILE SEPARATOR */
	0x1D, /* GROUP SEPARATOR */
	0x1E, /* RECORD SEPARATOR */
	0x1F, /* UNIT SEPARATOR */

	0x20, /* SPACE */
	0x21, /* EXCLAMATION MARK */
	0x22, /* QUOTATION MARK */
	0x23, /* NUMBER SIGN */
	0x24, /* DOLLAR SIGN */
	0x25, /* PERCENT SIGN */
	0x26, /* AMPERSAND */
	0x27, /* APOSTROPHE */
	0x28, /* LEFT PARENTHESIS */
	0x29, /* RIGHT PARENTHESIS */
	0x2A, /* ASTERISK */
	0x2B, /* PLUS SIGN */
	0x2C, /* COMMA */
	0x2D, /* HYPHEN-MINUS */
	0x2E, /* FULL STOP */
	0x2F, /* SOLIDUS */
	0x30, /* DIGIT ZERO */
	0x31, /* DIGIT ONE */
	0x32, /* DIGIT TWO */
	0x33, /* DIGIT THREE */
	0x34, /* DIGIT FOUR */
	0x35, /* DIGIT FIVE */
	0x36, /* DIGIT SIX */
	0x37, /* DIGIT SEVEN */
	0x38, /* DIGIT EIGHT */
	0x39, /* DIGIT NINE */
	0x3A, /* COLON */
	0x3B, /* SEMICOLON */
	0x3C, /* LESS-THAN SIGN */
	0x3D, /* EQUALS SIGN */
	0x3E, /* GREATER-THAN SIGN */
	0x3F, /* QUESTION MARK */

	0x40, /* COMMERCIAL AT */
	0x41, /* LATIN CAPITAL LETTER A */
	0x47, /* LATIN CAPITAL LETTER B */
	0x48, /* LATIN CAPITAL LETTER C */
	0x4C, /* LATIN CAPITAL LETTER D */
	0x4F, /* LATIN CAPITAL LETTER E */
	0x54, /* LATIN CAPITAL LETTER F */
	0x55, /* LATIN CAPITAL LETTER G */
	0x56, /* LATIN CAPITAL LETTER H */
	0x57, /* LATIN CAPITAL LETTER I */
	0x58, /* LATIN CAPITAL LETTER J */
	0x59, /* LATIN CAPITAL LETTER K */
	0x5A, /* LATIN CAPITAL LETTER L */
	0x5E, /* LATIN CAPITAL LETTER M */
	0x5F, /* LATIN CAPITAL LETTER N */
	0x62, /* LATIN CAPITAL LETTER O */
	0x67, /* LATIN CAPITAL LETTER P */
	0x68, /* LATIN CAPITAL LETTER Q */
	0x69, /* LATIN CAPITAL LETTER R */
	0x6C, /* LATIN CAPITAL LETTER S */
	0x70, /* LATIN CAPITAL LETTER T */
	0x73, /* LATIN CAPITAL LETTER U */
	0x78, /* LATIN CAPITAL LETTER V */
	0x79, /* LATIN CAPITAL LETTER W */
	0x7A, /* LATIN CAPITAL LETTER X */
	0x7B, /* LATIN CAPITAL LETTER Y */
	0x7D, /* LATIN CAPITAL LETTER Z */
	0x81, /* LEFT SQUARE BRACKET */
	0x82, /* REVERSE SOLIDUS */
	0x83, /* RIGHT SQUARE BRACKET */
	0x84, /* CIRCUMFLEX ACCENT */
	0x85, /* LOW LINE */

	0x60, /* GRAVE ACCENT */
	0x41, /* LATIN SMALL LETTER A */
	0x47, /* LATIN SMALL LETTER B */
	0x48, /* LATIN SMALL LETTER C */
	0x4C, /* LATIN SMALL LETTER D */
	0x4F, /* LATIN SMALL LETTER E */
	0x54, /* LATIN SMALL LETTER F */
	0x55, /* LATIN SMALL LETTER G */
	0x56, /* LATIN SMALL LETTER H */
	0x57, /* LATIN SMALL LETTER I */
	0x58, /* LATIN SMALL LETTER J */
	0x59, /* LATIN SMALL LETTER K */
	0x5A, /* LATIN SMALL LETTER L */
	0x5E, /* LATIN SMALL LETTER M */
	0x5F, /* LATIN SMALL LETTER N */
	0x62, /* LATIN SMALL LETTER O */
	0x67, /* LATIN SMALL LETTER P */
	0x68, /* LATIN SMALL LETTER Q */
	0x69, /* LATIN SMALL LETTER R */
	0x6C, /* LATIN SMALL LETTER S */
	0x70, /* LATIN SMALL LETTER T */
	0x73, /* LATIN SMALL LETTER U */
	0x78, /* LATIN SMALL LETTER V */
	0x79, /* LATIN SMALL LETTER W */
	0x7A, /* LATIN SMALL LETTER X */
	0x7B, /* LATIN SMALL LETTER Y */
	0x7D, /* LATIN SMALL LETTER Z */
	0x86, /* LEFT CURLY BRACKET */
	0x87, /* VERTICAL LINE */
	0x88, /* RIGHT CURLY BRACKET */
	0x89, /* TILDE */
	0x8A, /* DELETE */

	0x8B, /* <control> */
	0x8C, /* <control> */
	0x8D, /* <control> */
	0x8E, /* <control> */
	0x8F, /* <control> */
	0x90, /* <control> */
	0x91, /* <control> */
	0x92, /* <control> */
	0x93, /* <control> */
	0x94, /* <control> */
	0x95, /* <control> */
	0x96, /* <control> */
	0x97, /* <control> */
	0x98, /* <control> */
	0x99, /* <control> */
	0x9A, /* <control> */
	0x9B, /* <control> */
	0x9C, /* <control> */
	0x9D, /* <control> */
	0x9E, /* <control> */
	0x9F, /* <control> */
	0xA0, /* <control> */
	0xA1, /* <control> */
	0xA2, /* <control> */
	0xA3, /* <control> */
	0xA4, /* <control> */
	0xA5, /* <control> */
	0xA6, /* <control> */
	0xA7, /* <control> */
	0xA8, /* <control> */
	0xA9, /* <control> */
	0xAA, /* <control> */

	0xA0	,	/*	NO-BREAK SPACE */
	0xA1	,	/*	LEFT SINGLE QUOTATION MARK */
	0xA2,	/*	RIGHT SINGLE QUOTATION MARK */
	0xA3	,	/*	POUND SIGN */
	0xA4	,	/*	EURO SIGN */
	0xA5,	/*	DRACHMA SIGN */
	0xA6	,	/*	BROKEN BAR */
	0xA7	,	/*	SECTION SIGN */
	0xA8	,	/*	DIAERESIS */
	0xA9	,	/*	COPYRIGHT SIGN */
	0xAA	,	/*	GREEK YPOGEGRAMMENI */
	0xAB,	/*	LEFT-POINTING DOUBLE ANGLE QUOTATION MARK */
	0xAC,	/*	NOT SIGN */
	0xAD,	/*	SOFT HYPHEN */
	0xAF	,	/*	HORIZONTAL BAR */
	0xB0,	/*	DEGREE SIGN */
	0xB1,	/*	PLUS-MINUS SIGN */
	0xB2	,	/*	SUPERSCRIPT TWO */
	0xB3	,	/*	SUPERSCRIPT THREE */
	0xB4,	/*	GREEK TONOS */
	0xB5,	/*	GREEK DIALYTIKA TONOS */
	0xB7,	/*	MIDDLE DOT */
	0xBB,	/*	RIGHT-POINTING DOUBLE ANGLE QUOTATION MARK */
	0xBD,	/*	VULGAR FRACTION ONE HALF */

	0xC1,	/*	GREEK CAPITAL LETTER ALPHA */
	0xB6	,	/*	GREEK CAPITAL LETTER ALPHA WITH TONOS */
	0xC2,	/*	GREEK CAPITAL LETTER BETA */
	0xC3,	/*	GREEK CAPITAL LETTER GAMMA */
	0xC4,	/*	GREEK CAPITAL LETTER DELTA */
	0xC5,	/*	GREEK CAPITAL LETTER EPSILON */
	0xB8	,	/*	GREEK CAPITAL LETTER EPSILON WITH TONOS */
	0xC6,	/*	GREEK CAPITAL LETTER ZETA */
	0xC7,	/*	GREEK CAPITAL LETTER ETA */
	0xB9,	/*	GREEK CAPITAL LETTER ETA WITH TONOS */
	0xC8,	/*	GREEK CAPITAL LETTER THETA */
	0xC9,	/*	GREEK CAPITAL LETTER IOTA */
	0xBA	,	/*	GREEK CAPITAL LETTER IOTA WITH TONOS */
	0xDA,	/*	GREEK CAPITAL LETTER IOTA WITH DIALYTIKA */
	0xCA,	/*	GREEK CAPITAL LETTER KAPPA */
	0xCB,	/*	GREEK CAPITAL LETTER LAMDA */
	0xCC,	/*	GREEK CAPITAL LETTER MU */
	0xCD,	/*	GREEK CAPITAL LETTER NU */
	0xCE,	/*	GREEK CAPITAL LETTER XI */
	0xCF	,	/*	GREEK CAPITAL LETTER OMICRON */
	0xBC,	/*	GREEK CAPITAL LETTER OMICRON WITH TONOS */
	0xD0,	/*	GREEK CAPITAL LETTER PI */
	0xD1,	/*	GREEK CAPITAL LETTER RHO */
	0xD3,	/*	GREEK CAPITAL LETTER SIGMA */
	0xD4,	/*	GREEK CAPITAL LETTER TAU */
	0xD5,	/*	GREEK CAPITAL LETTER UPSILON */
	0xBE	,	/*	GREEK CAPITAL LETTER UPSILON WITH TONOS */
	0xDB,	/*	GREEK CAPITAL LETTER UPSILON WITH DIALYTIKA */
	0xD6,	/*	GREEK CAPITAL LETTER PHI */
	0xD7,	/*	GREEK CAPITAL LETTER CHI */
	0xD8,	/*	GREEK CAPITAL LETTER PSI */
	0xD9,	/*	GREEK CAPITAL LETTER OMEGA */
	0xBF	,	/*	GREEK CAPITAL LETTER OMEGA WITH TONOS */

	0xE1,	/*	GREEK SMALL LETTER ALPHA */
	0xDC,	/*	GREEK SMALL LETTER ALPHA WITH TONOS */
	0xE2	,	/*	GREEK SMALL LETTER BETA */
	0xE3,	/*	GREEK SMALL LETTER GAMMA */
	0xE4	,	/*	GREEK SMALL LETTER DELTA */
	0xE5,	/*	GREEK SMALL LETTER EPSILON */
	0xDD,	/*	GREEK SMALL LETTER EPSILON WITH TONOS */
	0xE6	,	/*	GREEK SMALL LETTER ZETA */
	0xE7,	/*	GREEK SMALL LETTER ETA */
	0xDE,	/*	GREEK SMALL LETTER ETA WITH TONOS */
	0xE8,	/*	GREEK SMALL LETTER THETA */
	0xE9,	/*	GREEK SMALL LETTER IOTA */
	0xDF,	/*	GREEK SMALL LETTER IOTA WITH TONOS */
	0xFA	,	/*	GREEK SMALL LETTER IOTA WITH DIALYTIKA */
	0xC0,	/*	GREEK SMALL LETTER IOTA WITH DIALYTIKA AND TONOS */
	0xEA	,	/*	GREEK SMALL LETTER KAPPA */
	0xEB	,	/*	GREEK SMALL LETTER LAMDA */
	0xEC,	/*	GREEK SMALL LETTER MU */
	0xED,	/*	GREEK SMALL LETTER NU */
	0xEE	,	/*	GREEK SMALL LETTER XI */
	0xEF	,	/*	GREEK SMALL LETTER OMICRON */
	0xFC	,	/*	GREEK SMALL LETTER OMICRON WITH TONOS */
	0xF0,	/*	GREEK SMALL LETTER PI */
	0xF1,	/*	GREEK SMALL LETTER RHO */
	0xF3	,	/*	GREEK SMALL LETTER SIGMA */
	0xF2	,	/*	GREEK SMALL LETTER FINAL SIGMA */
	0xF4,	/*	GREEK SMALL LETTER TAU */
	0xF5	,	/*	GREEK SMALL LETTER UPSILON */
	0xFB,	/*	GREEK SMALL LETTER UPSILON WITH DIALYTIKA */
	0xFD,	/*	GREEK SMALL LETTER UPSILON WITH TONOS */
	0xE0	,	/*	GREEK SMALL LETTER UPSILON WITH DIALYTIKA AND TONOS */
	0xF6,	/*	GREEK SMALL LETTER PHI */
	0xF7	,	/*	GREEK SMALL LETTER CHI */
	0xF8	,	/*	GREEK SMALL LETTER PSI */
	0xF9	,	/*	GREEK SMALL LETTER OMEGA */
	0xFE	,	/*	GREEK SMALL LETTER OMEGA WITH TONOS */
	0xB9  /* DOT ABOVE */
};

///
